"""Test preprocessing"""
import pathlib

import pytest

from nanite import IndentationGroup


datadir = pathlib.Path(__file__).resolve().parent / "data"
bad_files = list(datadir.glob("bad*"))


@pytest.mark.filterwarnings('ignore::nanite.preproc.CannotSplitWarning',
                            'ignore::UserWarning')
def test_process_bad():
    for bf in bad_files:
        ds = IndentationGroup(bf)
        for idnt in ds:
            # Walk through the standard analysis pipeline without
            # throwing any exceptions.
            idnt.apply_preprocessing(["compute_tip_position",
                                      "correct_force_offset",
                                      "correct_tip_offset",
                                      "correct_split_approach_retract"])
            idnt.fit_model()


def test_unknown_method():
    idnt = IndentationGroup(datadir / "spot3-0192.jpk-force")[0]
    try:
        idnt.apply_preprocessing(["compute_tip_position",
                                  "unknown_method"])
    except KeyError:
        pass
    else:
        assert False, "Preprocessing with an unknown method must not work."


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()



import pathlib

from .indent import Indentation
from .read import load_data


def load_group(path, callback=None):
    """Load indentation data from disk

    Parameters
    ----------
    path: path-like
        Path to experimental data
    callback: callable or None
        Callback function for tracking loading progress

    Returns
    -------
    group: nanite.IndetationGroup
        Indentation group with force-distance data
    """
    path = pathlib.Path(path)
    data = load_data(path, callback=callback)
    grp = IndentationGroup()
    for dd in data:
        grp.append(Indentation(dd))
    grp.path = path
    return grp


class IndentationGroup(object):
    def __init__(self, path=None, callback=None):
        """Group of Indentation

        Parameters
        ----------
        path: str
            The path to the data file. The data format is determined
            using the extension of the file and the data is loaded
            with the correct method.
        callback: callable or None
            A method that accepts a float between 0 and 1
            to externally track the process of loading the data.
        """
        self._mmlist = []

        if path is not None:
            self += load_group(path, callback=callback)

        self.path = path

    def __add__(self, grp):
        out = IndentationGroup()
        out._mmlist = self._mmlist + grp._mmlist
        return out

    def __iadd__(self, grp):
        self._mmlist += grp._mmlist
        self.path = None
        return self

    def __iter__(self):
        return iter(self._mmlist)

    def __getitem__(self, idx):
        return self._mmlist[idx]

    def __len__(self):
        return len(self._mmlist)

    def __repr__(self):
        rep = ["IndentationGroup: '{}'".format(self.path)]
        for idnt in self._mmlist:
            rep.append("- {}".format(idnt))
        return "\n".join(rep)

    def get_enum(self, enum):
        """Return the indentation curve with this enum value

        Raises
        ------
        ValueError if multiple curves with the same enum value exist.
        KeyErrir if the enum value is not found
        """
        curves = []
        for item in self._mmlist:
            if item.enum == enum:
                curves.append(item)
        if len(curves) == 0:
            raise KeyError("Could not find dataset with enum {}".format(enum))
        elif len(curves) == 1:
            return curves[0]
        else:
            raise ValueError("Multiple curves with the same enum value exist!")

    def append(self, item):
        """Append an indentation dataset

        Parameters
        ----------
        item: nanite.indent.Indentation
            Force-indentation dataset
        """
        if not isinstance(item, Indentation):
            raise ValueError("`item` must be an instance of `Indentation`!")
        self._mmlist.append(item)

    def index(self, item):
        return self._mmlist.index(item)

    def subgroup_with_path(self, path):
        """Return a subgroup with measurements matching `path`"""
        path = pathlib.Path(path)
        subgroup = IndentationGroup()
        for idnt in self:
            if pathlib.Path(idnt.path).resolve() == path.resolve():
                subgroup.append(idnt)
        subgroup.path = path
        return subgroup



import pathlib

import afmformats


def get_data_paths(path):
    """Obtain a list of data files

    Parameters
    ----------
    path: str or pathlib.Path
        Path to a data file or a directory containing
        data files.

    Returns
    -------
    paths: list of pathlib.Path
        All supported data files found in `path`. If `path`
        is a file, `[pathlib.Path(path)]` is returned. If
        `path` has an unsupported extion, an empty list is
        returned.
    """
    path = pathlib.Path(path)
    paths = []
    if path.is_dir():
        # recurse into directories
        for ext in afmformats.supported_extensions:
            paths += sorted(path.rglob("*{}".format(ext)))
    elif path.suffix in afmformats.supported_extensions:
        paths = [path]
    return sorted(paths)


def get_data_paths_enum(path, skip_errors=False):
    paths = get_data_paths(path)
    enumpaths = []
    for pp in paths:
        try:
            data = load_data(pp)
        except BaseException:
            if skip_errors:
                continue
            else:
                raise
        for dd in data:
            enumpaths.append([pp, dd.enum])
    return enumpaths


def load_data(path, callback=None):
    """Load data and return list of :class:`afmformats.AFMForceDistance`"""
    paths = get_data_paths(path)
    data = []
    for pp in paths:
        measurements = afmformats.load_data(pp, callback=callback)
        for dd in measurements:
            if dd.mode == "force-distance":
                data.append(dd)
    return data
